==> gnocl-0.5.15.tar.bz2: gnocl-0.5.15/src/canvas/canvasWidget.c <==
/*
 * $Id: canvasWidget.c,v 1.2 2003/09/01 18:13:08 baum Exp $
 *
 * This file implements the widget item of the canvas widget
 *
 * Copyright (c) 2001 - 2003 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2002-11: switched from GnoclWidgetOptions to GnoclOption
   2001-05: Begin of developement
 */

#include "canvas.h"
#include <string.h>
#include <assert.h>

static GnoclOption widgetOptions[] = 
{
   { "-coords", GNOCL_LIST, NULL },
   { "-widget", GNOCL_STRING, NULL },
   { "-anchor", GNOCL_OBJ, "anchor", gnoclOptAnchor }, 
   { "-width", GNOCL_DOUBLE, "width", NULL }, 
   { "-height", GNOCL_DOUBLE, "height", NULL }, 
   { "-onMotion", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onEnter", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onLeave", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonPress", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonRelease", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { NULL }
};

static const int coordsIdx = 0;
static const int widgetIdx = 1;

static int setOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info )
{
   gnoclResetSetOptions( widgetOptions );

   if( gnoclSetOptions( interp, widgetOptions, G_OBJECT( info->item ), -1 ) 
         != TCL_OK )
      return TCL_ERROR;

   if( gnoclCanvasSet2Double( interp, info->item, &widgetOptions[coordsIdx], 
         "x", "y" ) != TCL_OK )
      return TCL_ERROR;

   if( widgetOptions[widgetIdx].status == GNOCL_STATUS_CHANGED )
   {
      GtkWidget *widget = gnoclGetWidgetFromName( 
            widgetOptions[widgetIdx].val.str, interp );
      if( widget == NULL )
         return TCL_ERROR;

      gnome_canvas_item_set( info->item, "widget", widget, NULL );
   }

   return TCL_OK;
}

static int parseOptions( Tcl_Interp *interp, int objc, 
      Tcl_Obj * const objv[], Gnocl_CanvasItemInfo *info )
{
   gnoclClearOptions( widgetOptions );
   return gnoclParseOptions( interp, objc, objv, widgetOptions );
}

static int clearOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info ) 
{
   gnoclClearOptions( widgetOptions );
   return TCL_OK;
}

Gnocl_CanvasItemInfo *gnoclCanvasCreateWidget( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], GnomeCanvasGroup *group )
{
   Gnocl_CanvasItemInfo *info = g_new( Gnocl_CanvasItemInfo, 1 );
   info->parseOptions = &parseOptions;
   info->setOptions = &setOptions;
   info->clearOptions = &clearOptions;
   info->command = NULL;
   info->item = gnome_canvas_item_new( group, 
         gnome_canvas_widget_get_type( ), 
         "width", 100., "height", 100., NULL );

   /* init options */
   gnoclParseOptions( NULL, 0, NULL, widgetOptions );

   return info;
}

==> gnocl-0.5.18.tar.bz2: gnocl-0.5.18/src/canvas/canvasWidget.c <==
/*
 * $Id: canvasWidget.c,v 1.5 2004/06/28 17:51:18 baum Exp $
 *
 * This file implements the widget item of the canvas widget
 *
 * Copyright (c) 2001 - 2004 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2002-11: switched from GnoclWidgetOptions to GnoclOption
   2001-05: Begin of developement
 */

#include "canvas.h"
#include <string.h>
#include <assert.h>

static GnoclOption widgetOptions[] = 
{
   { "-widget", GNOCL_STRING, NULL },
   { "-anchor", GNOCL_OBJ, "anchor", gnoclOptAnchor }, 
   { "-coords", GNOCL_LIST, "?", gnoclOptXY },
   { "-height", GNOCL_DOUBLE, "height", NULL }, 
   { "-onButtonPress", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonRelease", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onEnter", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onLeave", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onMotion", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-parent", GNOCL_OBJ, "", gnoclOptParent },
   { "-width", GNOCL_DOUBLE, "width", NULL }, 
   { NULL }
};

static const int widgetIdx = 0;

static int setOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info )
{
   gnoclResetSetOptions( widgetOptions );

   if( gnoclSetOptions( interp, widgetOptions, G_OBJECT( info->item ), -1 ) 
         != TCL_OK )
      return TCL_ERROR;

   if( widgetOptions[widgetIdx].status == GNOCL_STATUS_CHANGED )
   {
      GtkWidget *widget = gnoclGetWidgetFromName( 
            widgetOptions[widgetIdx].val.str, interp );
      if( widget == NULL )
         return TCL_ERROR;

      gnome_canvas_item_set( info->item, "widget", widget, NULL );
   }

   return TCL_OK;
}

static Tcl_Obj *cget( Tcl_Interp *interp, int idx, Gnocl_CanvasItemInfo *info )
{
   Tcl_Obj *ret = NULL;

   if( idx == widgetIdx )
   {
      const char *name = NULL;
      GtkWidget  *widget;
      g_object_get( G_OBJECT( info->item ), "widget", &widget, NULL );
      if( widget )
         name = gnoclGetNameFromWidget( widget );
      
      if( name )
         ret = Tcl_NewStringObj( name, -1 );
      else
         ret = Tcl_NewStringObj( "", 0 );
   }
   
   return ret;
}

Gnocl_CanvasItemInfo *gnoclCanvasCreateWidget( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], GnomeCanvasGroup *group )
{
   Gnocl_CanvasItemInfo *info = g_new( Gnocl_CanvasItemInfo, 1 );
   info->options = widgetOptions;
   info->setOptions = &setOptions;
   info->getOption = &cget;
   info->command = NULL;
   info->item = gnome_canvas_item_new( group, 
         gnome_canvas_widget_get_type( ), 
         "width", 100., "height", 100., NULL );

   return info;
}
